<?xml version="1.0" encoding="UTF-8"?>
<!-- $Id: h2db.xsl,v 1.1.4.48 2008/10/23 12:43:36 robertDouglass Exp $ -->
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
 xmlns:html="http://www.w3.org/1999/xhtml" exclude-result-prefixes="xsl html">

 <xsl:output method="xml" indent="yes" omit-xml-declaration="yes"/>
 <xsl:param name="graphics_location"/>
 <xsl:param name="prefix"/>
 <xsl:param name="imagewidth.fo"/>
 <xsl:param name="imagewidth.html"/>
 <xsl:param name="imagealign.fo"/>
 <xsl:param name="imagealign.html"/>


 <!-- Main block-level conversions -->
 <xsl:template match="html:html">
  <xsl:apply-templates select="html:body"/>
 </xsl:template>

 <!-- This template converts each HTML file encountered into a DocBook 
     section.  For a title, it selects the first h1 element -->
 <xsl:template match="html:body">
  <xsl:apply-templates select="* | text()"/>
 </xsl:template>

 <!-- This template matches on all HTML header items and makes them into 
  bridgeheads. It attempts to assign an ID to each bridgehead by looking 
  for a named anchor as a child of the header or as the immediate preceding
  or following sibling -->
 <xsl:template
  match="html:body/html:h1 
                    | html:body/html:h2
                    | html:body/html:h3
                    | html:body/html:h4
                    | html:body/html:h5
                    | html:body/html:h6">

  <bridgehead>
   <xsl:choose>
    <xsl:when test="count(html:a/@name)">
     <xsl:attribute name="id">
      <xsl:value-of select="html:a/@name"/>
     </xsl:attribute>
    </xsl:when>
    <xsl:when test="preceding-sibling::* = preceding-sibling::html:a[@name != '']">
     <xsl:attribute name="id">
      <xsl:value-of select="concat($prefix,preceding-sibling::html:a[1]/@name)"/>
     </xsl:attribute>
    </xsl:when>
    <xsl:when test="following-sibling::* = following-sibling::html:a[@name != '']">
     <xsl:attribute name="id">
      <xsl:value-of select="concat($prefix,following-sibling::html:a[1]/@name)"/>
     </xsl:attribute>
    </xsl:when>
   </xsl:choose>
   <xsl:apply-templates/>
  </bridgehead>
 </xsl:template>

 <!-- These templates perform one-to-one conversions of HTML elements into
     DocBook elements -->
 <xsl:template match="html:p">
  <!-- if the paragraph has no text (perhaps only a child <img>), don't 
     make it a para -->
  <xsl:choose>
   <xsl:when test="normalize-space(.) = ''">
    <xsl:apply-templates/>
   </xsl:when>
   <xsl:otherwise>
    <para>
     <xsl:apply-templates/>
    </para>
   </xsl:otherwise>
  </xsl:choose>
 </xsl:template>
 <xsl:template match="html:blockquote">
  <blockquote>
   <para>
    <xsl:apply-templates/>
   </para>
  </blockquote>
 </xsl:template>
 <xsl:template match="html:pre">
  <programlisting>
  <xsl:apply-templates/>
 </programlisting>
 </xsl:template>
 <xsl:template match="html:code">
  <programlisting>
  <xsl:apply-templates/>
 </programlisting>
 </xsl:template>
 <xsl:template match="html:p/html:code">
  <literal>
   <xsl:apply-templates/>
  </literal>
 </xsl:template>
 <xsl:template match="html:a/html:code">
  <literal>
   <xsl:apply-templates/>
  </literal>
 </xsl:template>
 <xsl:template match="html:li/html:code">
  <literal>
   <xsl:apply-templates/>
  </literal>
 </xsl:template>

 <!-- Hyperlinks -->
 <xsl:template match="html:a[contains(@href,'http://')]" priority="1.5">
  <xsl:element name="ulink">
   <xsl:attribute name="url">
    <xsl:value-of select="normalize-space(@href)"/>
   </xsl:attribute>
   <xsl:apply-templates/>
  </xsl:element>
 </xsl:template>

 <xsl:template match="html:a[contains(@href,'#')]" priority="0.6">
  <xsl:element name="link">
   <xsl:attribute name="linkend">
    <xsl:call-template name="make_id">
     <xsl:with-param name="string" select="substring-after(@href,'#')"/>
    </xsl:call-template>
   </xsl:attribute>
   <xsl:value-of select="."/>
  </xsl:element>
 </xsl:template>

 <xsl:template match="html:a[@href != '']">
  <xsl:element name="link">
   <xsl:attribute name="linkend">
    <xsl:value-of select="$prefix"/>
    <xsl:call-template name="make_id">
     <xsl:with-param name="string" select="@href"/>
    </xsl:call-template>
   </xsl:attribute>
   <xsl:value-of select="."/>
  </xsl:element>
  <!-- Most of the time the <xsl:value-of select="."/> will
       take care of all the needs of the child nodes. In the
       case of dfn and acronym, however, extra processing is in
       order to handle indexterms, so apply those templates
       specifically. -->
  <xsl:apply-templates select="html:dfn | html:acronym"/>
 </xsl:template>

 <xsl:template match="html:a[@name != '']" priority="1">
  <xsl:element name="para">
   <xsl:attribute name="id">
    <xsl:call-template name="make_id">
     <xsl:with-param name="string" select="@name"/>
    </xsl:call-template>
   </xsl:attribute>
  </xsl:element>
  <xsl:value-of select="."/>
 </xsl:template>
 <!-- 
 <xsl:template match="html:a[@href != '' 
  and not(boolean(ancestor::html:p|ancestor::html:li))]" 
  priority="1">
  <para>
   <xref>
    <xsl:attribute name="linkend">
     <xsl:value-of select="$prefix"/>
     <xsl:text>_</xsl:text>
     <xsl:call-template name="make_id">
      <xsl:with-param name="string" select="@href"/>
     </xsl:call-template>
    </xsl:attribute>
   </xref>
  </para>
 </xsl:template>
 
 <xsl:template match="html:a[contains(@href,'#') 
  and not(boolean(ancestor::html:p|ancestor::html:li))]" 
  priority="1.1">
  <para>
   <xref>
    <xsl:attribute name="linkend">
     <xsl:value-of select="$prefix"/>
     <xsl:text>_</xsl:text>
     <xsl:call-template name="make_id">
      <xsl:with-param name="string" select="substring-after(@href,'#')"/>
     </xsl:call-template>
    </xsl:attribute>
   </xref>
  </para>
 </xsl:template>
-->
 <!-- Drupal specific conversion of URLs to IDs. 
      Removes /?q= from the beginnings, replaces / with -
   -->
 <xsl:template name="make_id">
  <xsl:param name="string" select="''"/>
  <xsl:param name="string2">
   <xsl:choose>
    <xsl:when test="starts-with($string, '/')">
     <xsl:value-of select="substring-after($string, '/')"/>
    </xsl:when>
    <xsl:otherwise>
     <xsl:value-of select="$string"/>
    </xsl:otherwise>
   </xsl:choose>
  </xsl:param>
  <xsl:call-template name="string.subst">
   <xsl:with-param name="string">
    <xsl:call-template name="string.subst">
     <xsl:with-param name="string" select="$string2"/>
     <xsl:with-param name="substitute">/?q=</xsl:with-param>
     <xsl:with-param name="with"/>
    </xsl:call-template>
   </xsl:with-param>
   <xsl:with-param name="substitute">/</xsl:with-param>
   <xsl:with-param name="with">-</xsl:with-param>
  </xsl:call-template>
 </xsl:template>

 <xsl:template name="string.subst">
  <xsl:param name="string" select="''"/>
  <xsl:param name="substitute" select="''"/>
  <xsl:param name="with" select="''"/>
  <xsl:choose>
   <xsl:when test="contains($string,$substitute)">
    <xsl:variable name="pre" select="substring-before($string,$substitute)"/>
    <xsl:variable name="post" select="substring-after($string,$substitute)"/>
    <xsl:call-template name="string.subst">
     <xsl:with-param name="string" select="concat($pre,$with,$post)"/>
     <xsl:with-param name="substitute" select="$substitute"/>
     <xsl:with-param name="with" select="$with"/>
    </xsl:call-template>
   </xsl:when>
   <xsl:otherwise>
    <xsl:value-of select="$string"/>
   </xsl:otherwise>
  </xsl:choose>
 </xsl:template>

 <!-- Glossary module support -->
 <xsl:template match="html:dfn | html:acronym">
  <xsl:call-template name="indexterm">
   <xsl:with-param name="primary" select="."/>
  </xsl:call-template>
  <!-- If dfn or acronym elements appear on their own (ie not in an <a> element)
       their child nodes should be processed normally. If they're in an <a>, however,
       the child nodes have already been processed as part of the <a> handling. -->
  <xsl:if test="parent[name()] != 'a'">
   <xsl:apply-templates/>
  </xsl:if>
 </xsl:template>

 <!-- Images -->
 <!-- Images and image maps -->
 <xsl:template match="html:img">
  <xsl:choose>
   <xsl:when
    test="boolean(parent::html:p) and 
        boolean(normalize-space(parent::html:p/text()))">
    <inlinemediaobject>
     <xsl:call-template name="process.inline-image"/>
    </inlinemediaobject>
   </xsl:when>
   <xsl:otherwise>
    <figure>
     <title>
      <xsl:if test="@alt != ''">
       <xsl:value-of select="@alt"/>
      </xsl:if>
     </title>
     <mediaobject>
      <xsl:call-template name="process.figure-image"/>
     </mediaobject>
    </figure>
   </xsl:otherwise>
  </xsl:choose>
 </xsl:template>

 <xsl:template name="process.inline-image">
  <imageobject role="html">
   <imagedata>
    <xsl:attribute name="fileref">
     <xsl:call-template name="make_absolute">
      <xsl:with-param name="filename" select="@src"/>
     </xsl:call-template>
    </xsl:attribute>
    <xsl:if test="@height != ''">
     <xsl:attribute name="depth">
      <xsl:value-of select="concat(@height,'px')"/>
     </xsl:attribute>
    </xsl:if>
    <xsl:if test="@width != ''">
     <xsl:attribute name="width">
      <xsl:value-of select="concat(@width,'px')"/>
     </xsl:attribute>
    </xsl:if>
   </imagedata>
  </imageobject>
  <imageobject role="fo">
   <imagedata>
    <xsl:attribute name="fileref">
     <xsl:call-template name="make_absolute">
      <xsl:with-param name="filename" select="@src"/>
     </xsl:call-template>
    </xsl:attribute>
    <xsl:if test="@height != ''">
     <xsl:attribute name="depth">
      <xsl:value-of select="concat(@height,'px')"/>
     </xsl:attribute>
    </xsl:if>
    <xsl:if test="@width != ''">
     <xsl:attribute name="width">
      <xsl:value-of select="concat(@width,'px')"/>
     </xsl:attribute>
    </xsl:if>
   </imagedata>
  </imageobject>
 </xsl:template>

 <xsl:template name="process.figure-image">
  <imageobject role="html">
   <imagedata>
    <xsl:attribute name="fileref">
     <xsl:call-template name="make_absolute">
      <xsl:with-param name="filename" select="@src"/>
     </xsl:call-template>
    </xsl:attribute>

    <xsl:attribute name="align">
     <xsl:value-of select="$imagealign.html"/>
    </xsl:attribute>
    <xsl:choose>
     <xsl:when test="number(@width) > $imagewidth.html">
      <xsl:attribute name="contentwidth">
       <xsl:value-of select="concat($imagewidth.html,'px')"/>
      </xsl:attribute>
     </xsl:when>
     <xsl:otherwise>
      <xsl:attribute name="contentwidth">
       <xsl:value-of select="concat(@width,'px')"/>
      </xsl:attribute>
     </xsl:otherwise>
    </xsl:choose>

   </imagedata>
  </imageobject>
  <imageobject role="fo">
   <imagedata>
    <xsl:attribute name="fileref">
     <xsl:call-template name="make_absolute">
      <xsl:with-param name="filename" select="@src"/>
     </xsl:call-template>
    </xsl:attribute>

    <xsl:attribute name="align">
     <xsl:value-of select="$imagealign.fo"/>
    </xsl:attribute>
    <xsl:choose>
     <xsl:when test="number(@width) > $imagewidth.fo">
      <xsl:attribute name="contentwidth">
       <xsl:value-of select="concat($imagewidth.fo,'px')"/>
      </xsl:attribute>
     </xsl:when>
     <xsl:otherwise>
      <xsl:attribute name="contentwidth">
       <xsl:value-of select="concat(@width,'px')"/>
      </xsl:attribute>
     </xsl:otherwise>
    </xsl:choose>

   </imagedata>
  </imageobject>
 </xsl:template>

 <xsl:template name="make_absolute">
  <xsl:param name="filename"/>
  <!-- 
 <xsl:variable name="name_only">
  <xsl:call-template name="get_filename">
   <xsl:with-param name="path" select="$filename"/>
  </xsl:call-template>
 </xsl:variable>
-->
  <xsl:value-of select="$graphics_location"/>
  <xsl:value-of select="$filename"/>
 </xsl:template>

 <xsl:template match="html:ul[count(*) = 0]">
  <xsl:message>Matched</xsl:message>
  <blockquote>
   <xsl:apply-templates/>
  </blockquote>
 </xsl:template>

 <xsl:template name="get_filename">
  <xsl:param name="path"/>
  <xsl:choose>
   <xsl:when test="contains($path,'/')">
    <xsl:call-template name="get_filename">
     <xsl:with-param name="path" select="substring-after($path,'/')"/>
    </xsl:call-template>
   </xsl:when>
   <xsl:otherwise>
    <xsl:value-of select="$path"/>
   </xsl:otherwise>
  </xsl:choose>
 </xsl:template>

 <!-- LIST ELEMENTS -->
 <xsl:template match="html:ul">
  <itemizedlist>
   <xsl:apply-templates/>
  </itemizedlist>
 </xsl:template>

 <!-- Some ol elements are meant to be index indicators. These have the class docbook-index. -->
 <xsl:template match="html:ol[contains(@class, 'docbook-index')]">
  <xsl:call-template name="indexterm">
   <xsl:with-param name="primary">
    <xsl:if test="html:li[1]">
     <xsl:value-of select="html:li[1]"/>
    </xsl:if>
   </xsl:with-param>
   <xsl:with-param name="secondary">
    <xsl:if test="html:li[2]">
     <xsl:value-of select="html:li[2]"/>
    </xsl:if>
   </xsl:with-param>
   <xsl:with-param name="tertiary">
    <xsl:if test="html:li[3]">
     <xsl:value-of select="html:li[3]"/>
    </xsl:if>
   </xsl:with-param>
  </xsl:call-template>
 </xsl:template>

 <xsl:template match="html:ol">
  <orderedlist>
   <xsl:apply-templates/>
  </orderedlist>
 </xsl:template>

 <xsl:template name="indexterm">
  <xsl:param name="primary"/>
  <xsl:param name="secondary"/>
  <xsl:param name="tertiary"/>
  <xsl:element name="indexterm">
   <xsl:if test="string-length($primary) > 0">
    <xsl:element name="primary">
     <xsl:value-of select="$primary"/>
    </xsl:element>
   </xsl:if>
   <xsl:if test="string-length($secondary) > 0">
    <xsl:element name="secondary">
     <xsl:value-of select="$secondary"/>
    </xsl:element>
   </xsl:if>
   <xsl:if test="string-length($tertiary) > 0">
    <xsl:element name="tertiary">
     <xsl:value-of select="$tertiary"/>
    </xsl:element>
   </xsl:if>
  </xsl:element>
 </xsl:template>

 <!-- DIV Handling:
      <div class="docbook-footnote|docbook-sidebar|docbook-tip|docbook-caution|docbook-warning|docbook-important|docbook-note">
        <h2>This is the title</h2>
        <p>Text goes here</>
        ...
      </div>
      
      This formation will become a Docbook XML footnote|sidebar|tip|caution|warning|important|note element. 
      If there is an adjacent child <h2> within the div, that will become the element's title.
 -->
 <xsl:template match="html:div">
  <xsl:choose>
   <xsl:when test="contains(@class, 'docbook-')">
    <!-- This measure is taken for class properties that contain several class names. -->
    <xsl:variable name="element_type">
     <xsl:call-template name="get_element_type">
      <xsl:with-param name="class" select="substring-after(@class, '-')"/>
     </xsl:call-template>
    </xsl:variable>
    <xsl:if
     test="$element_type = 'footnote' 
               or $element_type = 'sidebar'
               or $element_type = 'tip'
               or $element_type = 'caution'
               or $element_type = 'warning'
               or $element_type = 'important'
               or $element_type = 'note'">
     <xsl:element name="{$element_type}">
      <xsl:if test="@id">
       <xsl:attribute name="id">
        <xsl:value-of select="@id"/>
       </xsl:attribute>
      </xsl:if>
      <xsl:if test="html:h2[1]">
       <xsl:element name="title">
        <xsl:value-of select="html:h2[1]"/>
       </xsl:element>
      </xsl:if>
      <xsl:apply-templates/>
     </xsl:element>
    </xsl:if>
   </xsl:when>
   <xsl:otherwise>
    <xsl:apply-templates/>
   </xsl:otherwise>
  </xsl:choose>
 </xsl:template>

 <xsl:template name="wrap_para">
  <xsl:param name="text"/>
  <xsl:element name="para">
   <xsl:value-of select="normalize-space($text)"/>
  </xsl:element>
 </xsl:template>

 <xsl:template name="get_element_type">
  <xsl:param name="class"/>
  <xsl:choose>
   <xsl:when test="contains($class, ' ')">
    <xsl:call-template name="get_element_type">
     <xsl:with-param name="class" select="substring-before($class, ' ')"/>
    </xsl:call-template>
   </xsl:when>
   <xsl:otherwise>
    <xsl:value-of select="$class"/>
   </xsl:otherwise>
  </xsl:choose>
 </xsl:template>

 <!-- This template makes a DocBook variablelist out of an HTML definition list -->
 <xsl:template match="html:dl">
  <variablelist>
   <xsl:for-each select="html:dt">
    <varlistentry>
     <term>
      <xsl:apply-templates/>
     </term>
     <listitem>
      <xsl:apply-templates select="following-sibling::html:dd[1]"/>
     </listitem>
    </varlistentry>
   </xsl:for-each>
  </variablelist>
 </xsl:template>

 <xsl:template match="html:dd">
  <xsl:apply-templates/>
 </xsl:template>

 <xsl:template match="html:li">
  <listitem>
   <xsl:apply-templates/>
  </listitem>
 </xsl:template>



 <xsl:template
  match="html:h2[./parent::html:li] |
                      html:h3[./parent::html:li] |
                      html:h4[./parent::html:li] |
                      html:h5[./parent::html:li] |
                      html:h6[./parent::html:li]">
  <emphasis role="bold">
   <xsl:apply-templates/>
  </emphasis>
 </xsl:template>

 <xsl:template match="*">
  <xsl:message>No template for <xsl:value-of select="name()"/>
  </xsl:message>
  <xsl:apply-templates/>
 </xsl:template>

 <xsl:template match="@*">
  <xsl:message>No template for <xsl:value-of select="name()"/>
  </xsl:message>
  <xsl:apply-templates/>
 </xsl:template>

 <!-- inline formatting -->
 <xsl:template match="html:b | html:strong">
  <emphasis role="bold">
   <xsl:apply-templates/>
  </emphasis>
 </xsl:template>
 <xsl:template match="html:i | html:em">
  <emphasis>
   <xsl:apply-templates/>
  </emphasis>
 </xsl:template>
 <xsl:template match="html:u | html:cite">
  <citetitle>
   <xsl:apply-templates/>
  </citetitle>
 </xsl:template>

 <!-- Various elements -->
 <xsl:template match="html:hr"/>
 <xsl:template
  match="html:h1[1] |html:div/html:h2[1] |html:h3[1]"
  priority="1"/>
 <xsl:template match="html:br">
  <!-- don't change the indentation of <xsl:text> !!! -->
  <xsl:text>
</xsl:text>
 </xsl:template>
 <xsl:template match="html:span">
  <xsl:apply-templates/>
 </xsl:template>
 <xsl:template match="html:p[normalize-space(.) = '' and count(*) = 0]"/>
 <xsl:template match="html:tbody">
  <xsl:apply-templates select="*"/>
 </xsl:template>

 <!-- Table conversion -->
 <xsl:template match="html:table">
  <xsl:variable name="column_count">
   <xsl:call-template name="count_columns">
    <xsl:with-param name="table" select="."/>
   </xsl:call-template>
  </xsl:variable>
  <informaltable>
   <xsl:attribute name="border">
    <xsl:choose>
     <xsl:when test="@border = '' or @border = '0'">
      <xsl:value-of select="'0'"/>
     </xsl:when>
     <xsl:otherwise>
      <xsl:value-of select="@border"/>
     </xsl:otherwise>
    </xsl:choose>
   </xsl:attribute>
   <xsl:attribute name="frame">
    <xsl:choose>
     <xsl:when test="@border = '' or @border = '0'">
      <xsl:value-of select="'none'"/>
     </xsl:when>
     <xsl:otherwise>
      <xsl:value-of select="'all'"/>
     </xsl:otherwise>
    </xsl:choose>
   </xsl:attribute>
   <tgroup>
    <xsl:attribute name="cols">
     <xsl:value-of select="$column_count"/>
    </xsl:attribute>
    <xsl:call-template name="generate-colspecs">
     <xsl:with-param name="count" select="$column_count"/>
    </xsl:call-template>
    <thead>
     <xsl:apply-templates select="html:tr[1]"/>
    </thead>
    <tbody>
     <xsl:apply-templates select="html:tr[position() > 1]"/>
     <xsl:apply-templates select="html:tbody"/>
    </tbody>
   </tgroup>
  </informaltable>
 </xsl:template>

 <xsl:template name="generate-colspecs">
  <xsl:param name="count" select="0"/>
  <xsl:param name="number" select="1"/>
  <xsl:choose>
   <xsl:when test="$count &lt; $number"/>
   <xsl:otherwise>
    <colspec>
     <xsl:attribute name="colnum">
      <xsl:value-of select="$number"/>
     </xsl:attribute>
     <xsl:attribute name="colname">
      <xsl:value-of select="concat('col',$number)"/>
     </xsl:attribute>
    </colspec>
    <xsl:call-template name="generate-colspecs">
     <xsl:with-param name="count" select="$count"/>
     <xsl:with-param name="number" select="$number + 1"/>
    </xsl:call-template>
   </xsl:otherwise>
  </xsl:choose>
 </xsl:template>

 <xsl:template match="html:tr">
  <row>
   <xsl:apply-templates/>
  </row>
 </xsl:template>

 <xsl:template match="html:th|html:td">
  <xsl:variable name="position" select="count(preceding-sibling::*) + 1"/>
  <entry>
   <xsl:if test="@colspan &gt; 1">
    <xsl:attribute name="namest">
     <xsl:value-of select="concat('col',$position)"/>
    </xsl:attribute>
    <xsl:attribute name="nameend">
     <xsl:value-of select="concat('col',$position + number(@colspan) - 1)"/>
    </xsl:attribute>
   </xsl:if>
   <xsl:if test="@rowspan &gt; 1">
    <xsl:attribute name="morerows">
     <xsl:value-of select="number(@rowspan) - 1"/>
    </xsl:attribute>
   </xsl:if>
   <xsl:apply-templates/>
  </entry>
 </xsl:template>

 <xsl:template match="html:td_null">
  <xsl:apply-templates/>
 </xsl:template>

 <xsl:template name="count_columns">
  <xsl:param name="table" select="."/>
  <xsl:param name="row" select="$table//html:tr[1]"/>
  <xsl:param name="max" select="0"/>
  <xsl:choose>
   <xsl:when test="local-name($table) != 'table'">
    <xsl:message>Attempting to count columns on a non-table element</xsl:message>
   </xsl:when>
   <xsl:when test="local-name($row) != 'tr'">
    <xsl:message>Row parameter is not a valid row</xsl:message>
   </xsl:when>
   <xsl:otherwise>
    <!-- Count cells in the current row -->
    <xsl:variable name="current_count">
     <xsl:call-template name="count_cells">
      <xsl:with-param name="cell" select="$row/html:td[1]|$row/html:th[1]"/>
     </xsl:call-template>
    </xsl:variable>
    <!-- Check for the maximum value of $current_count and $max -->
    <xsl:variable name="new_max">
     <xsl:choose>
      <xsl:when test="$current_count &gt; $max">
       <xsl:value-of select="number($current_count)"/>
      </xsl:when>
      <xsl:otherwise>
       <xsl:value-of select="number($max)"/>
      </xsl:otherwise>
     </xsl:choose>
    </xsl:variable>
    <!-- If this is the last row, return $max, otherwise continue -->
    <xsl:choose>
     <xsl:when test="count($row/following-sibling::html:tr) = 0">
      <xsl:value-of select="$new_max"/>
     </xsl:when>
     <xsl:otherwise>
      <xsl:call-template name="count_columns">
       <xsl:with-param name="table" select="$table"/>
       <xsl:with-param name="row" select="$row/following-sibling::html:tr"/>
       <xsl:with-param name="max" select="$new_max"/>
      </xsl:call-template>
     </xsl:otherwise>
    </xsl:choose>
   </xsl:otherwise>
  </xsl:choose>
 </xsl:template>

 <xsl:template name="count_cells">
  <xsl:param name="cell"/>
  <xsl:param name="count" select="0"/>
  <xsl:variable name="new_count">
   <xsl:choose>
    <xsl:when test="$cell/@colspan &gt; 1">
     <xsl:value-of select="number($cell/@colspan) + number($count)"/>
    </xsl:when>
    <xsl:otherwise>
     <xsl:value-of select="number('1') + number($count)"/>
    </xsl:otherwise>
   </xsl:choose>
  </xsl:variable>
  <xsl:choose>
   <xsl:when test="count($cell/following-sibling::*) &gt; 0">
    <xsl:call-template name="count_cells">
     <xsl:with-param name="cell" select="$cell/following-sibling::*[1]"/>
     <xsl:with-param name="count" select="$new_count"/>
    </xsl:call-template>
   </xsl:when>
   <xsl:otherwise>
    <xsl:value-of select="$new_count"/>
   </xsl:otherwise>
  </xsl:choose>
 </xsl:template>

 <!-- text() handling. Some paragraphs in XHTML are really a collection of text nodes and inline elements.
      These templates look at each text node, determine what kind of parent and siblings it has, and
      inserts a [[para]] or [[/para]] placeholder where <para> or </para> should be. I couldn't find a way
      to get XSLT to insert <para> or </para> because they would be unmatched elements, and you can't do that. -->
 <xsl:template match="text()">
  <!-- This gets the parent element's name and stores it -->
  <xsl:variable name="parent_name">
   <xsl:for-each select="..">
    <xsl:value-of select="name()"/>
   </xsl:for-each>
  </xsl:variable>
  <xsl:choose>
   <!-- some docbook elements don't have <para>s as children. If we're dealing with
        one of the corresponding XHTML elements, don't do anything with this text. -->
   <xsl:when
    test="'p'  = $parent_name
                or 'h1' = $parent_name
                or 'h2' = $parent_name
                or 'h3' = $parent_name
                or 'h4' = $parent_name
                or 'h5' = $parent_name
                or 'h6' = $parent_name
                or 'dt' = $parent_name
                or 'code' = $parent_name
                or 'pre' = $parent_name">
    <!-- do nothing special, we like p et al. -->
    <xsl:copy-of select="."/>
   </xsl:when>
   <xsl:otherwise>
    <!-- We need to know if the parent is a block level element. -->
    <xsl:variable name="is_block_level">
     <xsl:call-template name="is-block-level">
      <xsl:with-param name="element_name" select="$parent_name"/>
     </xsl:call-template>
    </xsl:variable>
    <xsl:if test="'true' = $is_block_level">
     <!-- If there are no previous nodes, or if the immediately preceding is block level, print <para> -->
     <xsl:choose>
      <xsl:when test="count(preceding-sibling::* | preceding-sibling::text()) = 0">
       <xsl:text>[[para]]</xsl:text>
      </xsl:when>
      <xsl:otherwise>
       <xsl:for-each select="preceding-sibling::* | preceding-sibling::text()">
        <xsl:if test="position() = last()">
         <xsl:variable name="last_is_block_level">
          <xsl:call-template name="is-block-level">
           <xsl:with-param name="element_name" select="name()"/>
          </xsl:call-template>
         </xsl:variable>
         <!-- if the previous node was a block level element print [[para]] -->
         <xsl:if test="'true' = $last_is_block_level">
          <xsl:text>[[para]]</xsl:text>
         </xsl:if>
        </xsl:if>
       </xsl:for-each>
      </xsl:otherwise>
     </xsl:choose>
    </xsl:if>

    <!-- this prints the actual text. -->
    <xsl:copy-of select="."/>

    <xsl:if test="'true' = $is_block_level">
     <!-- If there are no following nodes, or if the immediate following is block level, print [[/para]] -->
     <xsl:choose>
      <xsl:when test="count(following-sibling::* | following-sibling::text()) = 0">
       <xsl:text>[[/para]]</xsl:text>
      </xsl:when>
      <xsl:otherwise>
       <xsl:for-each select="following-sibling::* | following-sibling::text()">
        <xsl:if test="position() = 1">
         <xsl:variable name="next_is_block_level">
          <xsl:call-template name="is-block-level">
           <xsl:with-param name="element_name" select="name()"/>
          </xsl:call-template>
         </xsl:variable>
         <xsl:if test="'true' = $next_is_block_level">
          <xsl:text>[[/para]]</xsl:text>
         </xsl:if>
        </xsl:if>
       </xsl:for-each>
      </xsl:otherwise>
     </xsl:choose>
    </xsl:if>
   </xsl:otherwise>
  </xsl:choose>
 </xsl:template>

 <xsl:template name="is-block-level">
  <xsl:param name="element_name"/>
  <xsl:choose>
   <xsl:when
    test="'address' = $element_name
    or 'body' = $element_name   
    or 'blockquote' = $element_name
    or 'br'         = $element_name
    or 'center'     = $element_name
    or 'dir'        = $element_name
    or 'div'        = $element_name
    or 'dl'         = $element_name
    or 'fieldset'   = $element_name
    or 'form'       = $element_name
    or 'h1'         = $element_name
    or 'h2'         = $element_name
    or 'h3'         = $element_name
    or 'h4'         = $element_name
    or 'h5'         = $element_name
    or 'h6'         = $element_name
    or 'hr'         = $element_name
    or 'isindex'    = $element_name
    or 'menu'       = $element_name
    or 'noframes'   = $element_name
    or 'noscript'   = $element_name
    or 'ol'         = $element_name
    or 'p'          = $element_name
    or 'pre'        = $element_name
    or 'table'      = $element_name
    or 'ul'         = $element_name
    or 'dd'         = $element_name
    or 'dt'         = $element_name
    or 'frameset'   = $element_name
    or 'li'         = $element_name
    or 'tbody'      = $element_name
    or 'td'         = $element_name
    or 'tfoot'      = $element_name
    or 'th'         = $element_name
    or 'thead'      = $element_name
    or 'tr'         = $element_name"
    >true</xsl:when>
   <xsl:otherwise>false</xsl:otherwise>
  </xsl:choose>
 </xsl:template>


</xsl:stylesheet>
